@@ -1,18 +1,18 @@
 #define QT_FEATURE_accessibility 1
 #define QT_FEATURE_action 1
-#define QT_FEATURE_opengles2 1
-#define QT_OPENGL_ES true
-#define QT_OPENGL_ES_2 true
+#define QT_FEATURE_opengles2 -1
+#define QT_FEATURE_dynamicgl -1
 #define QT_FEATURE_angle -1
 #define QT_FEATURE_clipboard 1
 #define QT_FEATURE_colornames 1
+#define QT_FEATURE_combined_angle_lib -1
 #define QT_FEATURE_cssparser 1
 #define QT_FEATURE_cursor 1
 #define QT_FEATURE_desktopservices 1
 #define QT_FEATURE_imageformat_xpm 1
 #define QT_FEATURE_draganddrop 1
-#define QT_FEATURE_dynamicgl -1
 #define QT_FEATURE_opengl 1
+#define QT_FEATURE_imageformatplugin 1
 #define QT_FEATURE_highdpiscaling 1
 #define QT_FEATURE_im 1
 #define QT_FEATURE_image_heuristic_mask 1
@@ -22,11 +22,10 @@
 #define QT_FEATURE_imageformat_png 1
 #define QT_FEATURE_imageformat_ppm 1
 #define QT_FEATURE_imageformat_xbm 1
-#define QT_FEATURE_imageformatplugin 1
 #define QT_FEATURE_movie 1
-#define QT_FEATURE_opengles3 1
-#define QT_OPENGL_ES_3 true
+#define QT_FEATURE_opengles3 -1
 #define QT_FEATURE_opengles31 -1
+#define QT_FEATURE_openvg -1
 #define QT_FEATURE_pdf 1
 #define QT_FEATURE_picture 1
 #define QT_QPA_DEFAULT_PLATFORM_NAME "xcb"
